#ifndef COTEXNODE_H
#define COTEXNODE_H

#include <qstring.h>

#include <Vector3D.h>

#include "AttributeInterface.h"
#include "HyperFunable.h"
#include "Savable.h"


/// The base classe for the CoTeXTree's nodes.
/**
 * CoTeXTrees are binary trees. Each node is able to give its left child,
 * its right child, its father, and its name (useful for saving/exporting). 
 */
class CoTeXNode : public HyperFunable, public Savable{
public : 

  /// The base constructor for CoTeXNodes.
  /**
   * The name, in some cases, must be unique : this constraint is resolved by the 
   * StringTool of PointShop, in the inherited class and/or in managers.
   */
  CoTeXNode (QString n) : name (n), lChild (NULL), rChild (NULL), father (NULL) {} 
  virtual ~CoTeXNode () {}

  /// For recursive destruction of a tree.
  void destroyChildren ();

  // *********
  // Accessors
  // *********
  
  inline CoTeXNode * getLeftChild () const { return lChild;}
  inline CoTeXNode * getRightChild () const { return rChild;}
  inline CoTeXNode * getFather () const { return father;}
  inline QString getName () const { return name;}
  
  void setLeftChild (CoTeXNode * n);
  void setRightChild (CoTeXNode * n);
  void setFather (CoTeXNode * n);

  /// newName must be non-null and non-empty.
  void setName (QString newName);
  
  /// Value returned by the node (Constructive texturing evaluation).
  /**
   * Compute the attribute at this position of the space.
   *
   * @param position the 3D point evaluated.
   * @return AttributeInterface*
   *         the value at <code>position</code> according to this node, 
   *         NULL if there's not.
   */
  virtual const AttributeInterface * getAttribute (const Vector3D& position) = 0;
  
  
  /// F-Rep function.
  /**
   * F-Rep evaluation of a 3D position.
   *
   * @param position the evaluated position.
   * @return double
   *         negative if the point is outside the primitive, 
   *         positive if it's inside
   *         0 on the surface.
   */
  virtual double eval (const Vector3D& position) = 0;
  
  /// Validity of a node
  /**
   * Check the validity of nides. For example, some inherited classes have 
   * to be leaf or internal node.
   */
  virtual bool check () = 0;

private :
  QString name;
  CoTeXNode * lChild; ///< the left child, NULL if there's not
  CoTeXNode * rChild; ///< the right child, NULL if there's not
  CoTeXNode * father; ///< the father, NULL if there's not
};


#endif // COTEXNODE_H
