#include "CoTeXTreeInterface.h"
#include <assert.h>
#include <Scene.h>
#include <StatusBar.h>

CoTeXTreeInterface::CoTeXTreeInterface (AttributeInterface * attribute) 
    : root (NULL), current (NULL) {
  assert (attribute != NULL);
  defaultAttribute = attribute;
}

CoTeXTreeInterface::CoTeXTreeInterface (const CoTeXTreeInterface & tree)
  : HyperFunable (), root (NULL), current (NULL), defaultAttribute (tree.defaultAttribute) {}
  
CoTeXTreeInterface::~CoTeXTreeInterface () {
  delete defaultAttribute;
  if (root != NULL) {
    root->destroyChildren ();
    delete root;
  }
}
    
void CoTeXTreeInterface::setRoot (CoTeXNode * newRoot) { 
  root = newRoot;
  current = root;
  
  // Place the current pointer for adding primtive in the future.
  if (current != NULL)
    while (current->getLeftChild () != NULL)
      current = current->getLeftChild ();
}

bool CoTeXTreeInterface::contains (PrimitiveInterface * p) const {
  assert (p != NULL);
  CoTeXNode * finder;
  for (finder = p; finder != NULL; finder = finder->getFather ())
    if (finder == root)
      return true;
  return false;
}

void CoTeXTreeInterface::addPrimitive (PrimitiveInterface * p, 
				       OperatorInterface * op) {
  assert (p != NULL && op != NULL);
  if (root == NULL) {
    root = current = p;
    current->setFather (NULL);
    delete op; // op is not useful
  }
  else {
    CoTeXNode * f = current->getFather ();
    op->setLeftChild (p);
    p->setFather (op);
    op->setRightChild (current);
    current->setFather (op);
    op->setFather (f);
    if (f != NULL)
      (f->getLeftChild () == current) ? f->setLeftChild (op) : f->setRightChild (op);
    else
      root = op;
    current = p;
  }
}
    
PrimitiveInterface * CoTeXTreeInterface::removePrimitive (PrimitiveInterface * p) {
  assert (!isEmpty () && contains (p));
  CoTeXNode * father = p->getFather ();
  
  if (father == NULL) // only 1 primitive in the tree
    root = current = NULL;
  else {
    CoTeXNode * grandFather = father->getFather ();
    CoTeXNode * brother = 
      (father->getLeftChild () == p) ? 
      father->getRightChild () : father->getLeftChild ();
    
    if (grandFather == NULL) { // only 2 primitives in the tree
      delete father; // = the root
      root = current = brother;
      brother->setFather (NULL);
    }
    else {
      if (p == current)
	current = brother; // current mustn't be NULL if the tree isn't empty
      brother->setFather (grandFather);
      (grandFather->getLeftChild () == father) ? 
	grandFather->setLeftChild (brother) : grandFather->setRightChild (brother);
      delete father;
    }
  }
  return p;
}
  
void CoTeXTreeInterface::apply (SurfelCollection * collection) {
  Scene * scene = Scene::getInstance ();
  SurfelInterface * surfel;
  unsigned int i, nofSurfels = collection->getNofSurfels () - 1; // the last updates the view
  StatusBar * statusBar = StatusBar::getInstance (); // informations for users
  
  statusBar->showMessage ("CoTeX computing", 0);
  for (i = 0, surfel = collection->getFirstSurfel ();
       i <= nofSurfels; 
       i++, surfel = collection->getNextSurfel ()) {
    // 3rd param. of affectSurfel : update view only for last 
    if (getRoot () != NULL) {
      const AttributeInterface * attribut = 
	getRoot ()->getAttribute (surfel->getPosition ());
      if (attribut != NULL)
	attribut->affectSurfel (surfel, scene, (i == nofSurfels ? true : false)); 
      else
	defaultAttribute->affectSurfel (surfel, scene, (i == nofSurfels ? true : false)); 
    }
    else
      defaultAttribute->affectSurfel (surfel, scene, (i == nofSurfels ? true : false)); 
	 
    if (i % (nofSurfels / 100) == 0)
      statusBar->setProgress ((int)((float)i / (float)nofSurfels * 100.0f), false);
  }
  statusBar->showMessage ("Done.", 0);
  statusBar->setProgress (100);
}
  
void CoTeXTreeInterface::apply (Object * object) {
  apply (object->getSurfelCollection ());
}

// *********
// Protected
// *********

void CoTeXTreeInterface::setDefaultAttribute (AttributeInterface * newDefault) {
  assert (newDefault != NULL);
  defaultAttribute = newDefault;
}
