#ifndef PRIMITIVEMANAGERINTERFACE_H
#define PRIMITIVEMANAGERINTERFACE_H

#include <qmap.h>
#include <Object.h>
#include <qstring.h>
#include "PrimitiveInterface.h"
#include "../Attributes/ColorAttribute.h"
#include "../../ui/CoTeXConfigurationWidget.h"


/// Exception threw by a bad set events for primitive creation.
class  PrimitiveManagerException {
public :

  PrimitiveManagerException (QString msg) : message (msg) {}
  inline QString getMessage () const { return message;}

private : 

  QString message;
};
    

/// The base class for the creation of primitives by user's events.
/**
 * PrimtiveManagers provide differents service : 
 *  - handle the UI events redirected by CoTeXTool, according to its current 
 *    configuration, for the interactive creation of the primtives.
 *  - give the pritimive created
 *  - give the PointShop Object repesenting this primitive.  
 */
class PrimitiveManagerInterface {
public :

  /// Map for quering PrimitiveManagers with primitive's names (like "sphere", "RBF", etc)
  typedef QMap<QString,PrimitiveManagerInterface*> PrimitiveManagerMap;
  /// Map for quering generated objects with their base primitives.
  typedef QMap<PrimitiveInterface*,Object*> PrimitiveObjectMap;
  /// Map for quering the set of CoTeX 's Primitives Objects for a given Object
  typedef QMap<Object*, PrimitiveObjectMap*> ObjectObjects;
 
  /** 
   * Return the primitive previously created, 
   * NULL if there's not or if it has been previously gived.
   */
  virtual PrimitiveInterface * getCurrentPrimitive () = 0;
  
  
  /**
   * Return the object based on the last current primitive, 
   * NULL if there's not or if it has been previously gived.
   */
  virtual Object * getCurrentObject () = 0;
  
  /// Create a primitive, as simple as possible
  /**
   * The primitive is made the minimum features.
   *
   * @return a PrimitiveInterface, the primitive created
   */
  virtual PrimitiveInterface * createPrimitive () = 0;
  
  /// Logic linked to the MousePressEvent
  /**
   * @param event the UI event from PointShop.
   * @param confWidget by convention, the press event is the first event for the creation 
   *                of a primitive, so the manager need some informations on the current
   *                of CoTeX : the CoTeXConfigurationWidget provide it.
   */
  virtual void handleMousePressEvent (QMouseEvent *event, 
				      CoTeXConfigurationWidget * confWidget) = 0;

  /// Logic linked to the MouseMovedEvent
  /**
   * @param event the UI event from PointShop.
   */
  virtual void handleMouseMoveEvent (QMouseEvent *event) = 0;

  /// Logic linked to the MouseReleaseEvent
  /**
   * @param event the UI event from PointShop.
   */
  virtual void handleMouseReleaseEvent (QMouseEvent *) = 0;
  
  /// Logic linked to the KeyPressedEvent
  /**
   * @param event the UI event from PointShop.
   */
  virtual void handleKeyPressEvent (QKeyEvent *ke) = 0;
  

  /// Logic linked to the KeyReleaseEvent
  /**
   * @param event the UI event from PointShop.
   */
  virtual void handleKeyReleaseEvent (QKeyEvent *ke) = 0;
};

#endif // PRIMITIVEMANAGERINTERFACE_H
