#ifndef RBF_H
#define RBF_H

#include <vector>
#include <assert.h>
#include <Vector3D.h>
#include "../Interfaces/PrimitiveInterface.h"
#include "../Interfaces/TextureInterface.h"
#include "../Textures/SimpleColorTexture.h"

static const unsigned int RBF_MIN_POINTSET_SIZE = 3; ///< the lowest for having a valid Rbf
static const unsigned int RBF_MAX_POINTSET_SIZE = 200;  ///< the highest number of points for defining a valid Rbf in this implementation.

//Constants needed for the aplication of Linear Solver.
static const int RBF_SURFACE_ISOVALUE = 0;
static const int RBF_NORMAL_ISOVALUE = -1;

/// A Rbf, specified by a number of points in the space
class Rbf : public PrimitiveInterface {
public:
  
  typedef std::vector<Vector3D> RbfSet; 
  /// General constructor for Rbf
  /**
   * @param n
   *        the name of the <code>Rbf</code>.
   * @param p 
   *        an QVector, which has the points describing the Rbf
   * @param t
   *        the texture carried by this primitive.
   * @param threshold
   *        the threshold
   */
  Rbf (QString n, RbfSet * rbfPointSet, 
       TextureInterface * tex = DEFAULT_SIMPLECOLOR);
  
  Rbf (Rbf &rbf); 
  virtual ~Rbf () {} 

  inline RbfSet* getRbfPointSet (){ return rbfPointSet;}
  void setRbfPointSet (RbfSet * newRbfPointSet);
  
  inline double* getResults () { return results;};
  
  /**
   *Redefinition of the operator = for the class <code>Rbf</code>
   *@param rbf
   *         an <code>Rbf</code>
   *@return Rbf
   *         the present object with the new values
   */
  Rbf& operator= (Rbf& rbf);

  /// Evaluation of the Radial Basis Fonction
  /**
   * Evaluate the value of the radial basis function. For having the surface 
   * (null values), we substract the threshold of all evaluations. As the function decrease
   * exponentially, we need a very small threshold for having good results.
   * 
   * @return double.
   */
  double eval (const Vector3D& position);
  
  BBoxMinMax getBBoxMinMax () { return bBoxMinMax;}

  // **********************
  // Interface HyperFunable
  // **********************
  
  void writeHFString (QTextStream & outStream);
  void writeDefArray (QTextStream & outStream);
  void writeDefBody (QTextStream & outStream);
  
  
  // *****************
  // Interface Savable
  // *****************
  
  void save (QTextStream &stream);
  void load (QTextStream &stream);


 private:
  RbfSet * rbfPointSet; ///< points describing the Rbf structure
  double *results; ///< array of the lambda results of the Radial Basis Function
  BBoxMinMax bBoxMinMax; ///<Bounding box of the Rbf structure
  
  /// Used when point set is changed, for computing good results
  void init ();
  
  /**
   * Creates a bounding box from the points describing the Rbf structure
   * @return BBoxMinMax   
   */
  BBoxMinMax computeBBoxMinMax ();

  /**
   * @return double
   *        the result value of applying the phi function to x, y, z.
   */
  double phi (double x, double y, double z);

  /**
   * Adds the new points to the Rbf structure 
   */
  void rbfReconstruction ();  

  
  /**
   * Computes the matrix needed for the utilisation of the Linear Solver with 
   * all the points describing the Rbf. After having created the matrix, calls 
   * the static method LinearSolver::solve()
   * @see #LinearSolver
   */  
  void solve();

};

#endif // RBF_H

