#ifndef SURFELGENERATOR_H
#define SURFELGENERATOR_H

#include <Object.h>

#include "../Attributes/ColorAttribute.h"
#include "../Interfaces/PrimitiveInterface.h"
#include "../BBoxMinMax/BBoxMinMax.h"

static const float COTEX_SURFEL_RADIUS = 1.0; ///< radius for cotex primitive's surfels 
static const ColorAttribute COTEX_SURFEL_COLOR = ColorAttribute (255, 255, 255); ///< the base color of cotex primitive objects.
static const Vector3D VIZ_DEFAULT_POSITION = Vector3D (0.0, 0.0, 0.0);

const int SPHERE_DEFAULT_RESOLUTION = 16; ///< resolution of generated spheres


/// Generate a point cloud for a CoTeX Primitive.
/**
 * Fill the primitive's Object with a set of surfels representing the primitive.
 * These surfels are computed in the limit of the bounginBox.
 * The function-based version of the HyperFun Polygonizer is used.
 *
 * @param primitive the primitive to represent, using its F-Rep evaluation.
 * @param primitiveObject the object to fill with the computed surfels.
 * @param bBoxMinMax the BoundingBox in which the surfels will be computed.
 * @param surfelColor the color of the created surfels.
 * 
 * @see #PrimitiveInterface
 */
extern void generatePrimitiveSurfels (PrimitiveInterface * primitive, 
				      Object * primitiveObject, 
				      const BBoxMinMax & bBoxMinMax,
				      QColor surfelColor = QColor (255, 255, 255));

/**
 * Set an object (with its SurfelCollection) to be a point-based sphere, centered
 * on (0,0,0), enough detailled for visualization of partitions (the primitives).
 *
 * It can be used for fast visualisation of complexes primitives which can be 
 * recovered by spheres, or for debug versions.
 *
 * @param object the object to fill
 * @param radius the sphere radius, whiwh must be positive
 * @param surfelColor the color of created surfels
 * @param meridien number of point on a vertical section of the half sphere.
 * @param parallele number of point on an horizontal section of the half sphere.
 * 
 */
extern void generateSphereSurfels (Object * object,
				   float radius, 
				   const ColorAttribute & surfelColor = COTEX_SURFEL_COLOR,
				   unsigned int meridien = SPHERE_DEFAULT_RESOLUTION, 
				   unsigned int parallele = SPHERE_DEFAULT_RESOLUTION);



/// Generate an Object representing the primitive, based on the PointShop object's mark
/**
 * @param primitive the primitive used for generating the new object
 * @param pointShopObject the new object must be expressed through the object textured
 *
 * @return an Object* the new object created
 */
extern Object * generateObject (PrimitiveInterface * primitive, Object * pointShopObject);

 /// Generate a new color, different from the texture, for better viz.
  /**
   * Sometimes, we need to have surfel's color different from the object one.
   * This function try to define it.
   *
   * @param texture the texture we want to extract a differetn color.
   * @param position the point used for the texture evaluation
   * @return ColorAttribute the vizualisation color
   */
extern ColorAttribute createColorForViz (TextureInterface * texture,
					 const Vector3D & position = VIZ_DEFAULT_POSITION);


#endif // SURFELGENERATOR_H
